@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -128,10 +128,11 @@ function cleanPreviousTestResults() {
 
 	jQuery('[id^="preproc-test-step-"][id$="-result"]', $form).empty();
 	jQuery('[id^="preproc-test-step-"][id$="-name"] > div', $form).remove();
-	jQuery('#final-result', $form)
+	jQuery('.js-final-result', $form)
 		.hide()
-		.find('.table-forms-td-right')
-		.empty();
+		.next()
+		.empty()
+		.hide();
 }
 
 /**
@@ -140,7 +141,8 @@ function cleanPreviousTestResults() {
  * @param {object} overlay  Overlay dialog object.
  */
 function itemGetValueTest(overlay) {
-	var $form = overlay.$dialogue.find('form'),
+	var $body = overlay.$dialogue.$body,
+		$form = overlay.$dialogue.find('form'),
 		form_data = $form.serializeJSON(),
 		post_data = getItemTestProperties('#preprocessing-test-form'),
 		interface = (typeof form_data['interface'] !== 'undefined') ? form_data['interface'] : null,
@@ -153,7 +155,8 @@ function itemGetValueTest(overlay) {
 			address: interface ? interface['address'].trim() : '',
 			port: (interface && interface['port']) ? interface['port'].trim() : '',
 			interfaceid: interface ? interface['interfaceid'] : null,
-			useip: interface ? interface['useip'] : null
+			useip: interface ? interface['useip'] : null,
+			details: interface ? interface['details'] : null
 		},
 		macros: form_data['macros'],
 		proxy_hostid: form_data['proxy_hostid'],
@@ -186,26 +189,29 @@ function itemGetValueTest(overlay) {
 			overlay.unsetLoading();
 		},
 		success: function(ret) {
-			overlay.$dialogue.find('.msg-bad, .msg-good').remove();
+			overlay.$dialogue.find('.msg-bad, .msg-good, .msg-warning').remove();
 
-			if (typeof ret.messages !== 'undefined') {
-				jQuery(ret.messages).insertBefore($form);
+			if ('error' in ret) {
+				const message_box = makeMessageBox('bad', ret.error.messages, ret.error.title);
+
+				jQuery($body).prepend(message_box);
+
+				return;
 			}
-			else {
-				<?php if ($data['show_prev']): ?>
-					if (typeof ret.prev_value !== 'undefined') {
-						jQuery('#prev_value', $form).multilineInput('value', ret.prev_value);
-						jQuery('#prev_time', $form).val(ret.prev_time);
-						jQuery('#upd_prev', $form).val(form_data['upd_last']);
-						jQuery('#upd_last', $form).val(Math.ceil(+new Date() / 1000));
-					}
-				<?php endif ?>
 
-				jQuery('#value', $form).multilineInput('value', ret.value);
-
-				if (typeof ret.eol !== 'undefined') {
-					jQuery("input[value=" + ret.eol + "]", jQuery("#eol")).prop("checked", "checked");
+			<?php if ($data['show_prev']): ?>
+				if (typeof ret.prev_value !== 'undefined') {
+					jQuery('#prev_value', $form).multilineInput('value', ret.prev_value);
+					jQuery('#prev_time', $form).val(ret.prev_time);
+					jQuery('#upd_prev', $form).val(form_data['upd_last']);
+					jQuery('#upd_last', $form).val(Math.ceil(+new Date() / 1000));
 				}
+			<?php endif ?>
+
+			jQuery('#value', $form).multilineInput('value', ret.value);
+
+			if (typeof ret.eol !== 'undefined') {
+				jQuery("input[value=" + ret.eol + "]", jQuery("#eol")).prop("checked", "checked");
 			}
 		},
 		dataType: 'json',
@@ -219,7 +225,8 @@ function itemGetValueTest(overlay) {
  * @param {object} overlay  Overlay dialog object.
  */
 function itemCompleteTest(overlay) {
-	var $form = overlay.$dialogue.find('form'),
+	var $body = overlay.$dialogue.$body,
+		$form = overlay.$dialogue.find('form'),
 		form_data = $form.serializeJSON(),
 		post_data = getItemTestProperties('#preprocessing-test-form'),
 		interface = (typeof form_data['interface'] !== 'undefined') ? form_data['interface'] : null,
@@ -234,7 +241,8 @@ function itemCompleteTest(overlay) {
 			address: interface ? interface['address'].trim() : '',
 			port: (interface && interface['port']) ? interface['port'].trim() : '',
 			interfaceid: interface ? interface['interfaceid'] : null,
-			useip: interface ? interface['useip'] : null
+			useip: interface ? interface['useip'] : null,
+			details: interface ? interface['details'] : null
 		},
 		macros: form_data['macros'],
 		proxy_hostid: form_data['proxy_hostid'],
@@ -242,7 +250,8 @@ function itemCompleteTest(overlay) {
 		test_type: <?= $data['test_type'] ?>,
 		hostid: <?= $data['hostid'] ?>,
 		valuemapid: <?= $data['valuemapid'] ?>,
-		value: form_data['value']
+		value: form_data['value'],
+		not_supported: form_data['not_supported']
 	});
 
 	<?php if ($data['show_prev']): ?>
@@ -271,13 +280,15 @@ function itemCompleteTest(overlay) {
 			overlay.unsetLoading();
 		},
 		success: function(ret) {
-			overlay.$dialogue.find('.msg-bad, .msg-good').remove();
+			overlay.$dialogue.find('.msg-bad, .msg-good, .msg-warning').remove();
 
-			if (typeof ret.messages !== 'undefined') {
-				jQuery(ret.messages).insertBefore($form);
+			if ('error' in ret) {
+				const message_box = makeMessageBox('bad', ret.error.messages, ret.error.title);
+
+				jQuery($body).prepend(message_box);
 			}
 
-			processItemPreprocessingTestResults(ret.steps);
+			processItemPreprocessingTestResults(ret.steps ?? []);
 
 			<?php if ($data['show_prev']): ?>
 				if (typeof ret.prev_value !== 'undefined') {
@@ -301,8 +312,9 @@ function itemCompleteTest(overlay) {
 				}
 
 				$result_row = jQuery('<div>', {'class': '<?= ZBX_STYLE_TABLE_FORMS_SEPARATOR ?>'})
+					.css({whiteSpace: 'normal'})
 					.append(jQuery('<div>').append(ret.final.action, $result))
-					.css('display', 'block');
+					.css({display: 'block', width: '675px'});
 
 				if (typeof ret.mapped_value !== 'undefined') {
 					$mapped_value = makeStepResult({result: ret.mapped_value});
@@ -317,10 +329,11 @@ function itemCompleteTest(overlay) {
 					);
 				}
 
-				jQuery('#final-result')
+				jQuery('.js-final-result')
 					.show()
-					.find('.table-forms-td-right')
-					.append($result_row);
+					.next()
+					.append($result_row)
+					.show();
 			}
 		},
 		dataType: 'json',
@@ -388,6 +401,8 @@ function saveItemTestInputs() {
 			value: jQuery('#value').multilineInput('value'),
 			eol: jQuery('#eol').find(':checked').val()
 		},
+		form_data = $form.serializeJSON(),
+		interface = (typeof form_data['interface'] !== 'undefined') ? form_data['interface'] : null,
 		macros = {};
 
 	<?php if ($data['is_item_testable']): ?>
@@ -396,7 +411,8 @@ function saveItemTestInputs() {
 			proxy_hostid: jQuery('#proxy_hostid', $form).val(),
 			interfaceid: <?= $data['interfaceid'] ?> || 0,
 			address: jQuery('#interface_address', $form).val(),
-			port: jQuery('#interface_port', $form).val()
+			port: jQuery('#interface_port', $form).val(),
+			interface_details: (interface && 'details' in interface) ? interface['details'] : null
 		});
 	<?php endif ?>
 
@@ -416,7 +432,7 @@ function saveItemTestInputs() {
 	<?php if ($data['step_obj'] == -2): ?>
 		$test_obj = jQuery('.tfoot-buttons');
 	<?php elseif ($data['step_obj'] == -1): ?>
-		$test_obj = jQuery('preprocessing-list-foot', jQuery('#preprocessing'));
+		$test_obj = jQuery('.preprocessing-list-foot', jQuery('#preprocessing'));
 	<?php else: ?>
 		$test_obj = jQuery('.preprocessing-list-item[data-step=<?= $data['step_obj'] ?>]', jQuery('#preprocessing'));
 	<?php endif ?>
@@ -425,7 +441,7 @@ function saveItemTestInputs() {
 }
 
 jQuery(document).ready(function($) {
-	$('#final-result').hide();
+	$('.js-final-result').hide().next().hide();
 
 	<?php if ($data['show_prev']): ?>
 		jQuery('#upd_last').val(Math.ceil(+new Date() / 1000));
@@ -435,7 +451,6 @@ jQuery(document).ready(function($) {
 		placeholder: <?= json_encode(_('value')) ?>,
 		value: <?= json_encode($data['value']) ?>,
 		monospace_font: false,
-		maxlength: 65535,
 		autofocus: true,
 		readonly: false,
 		grow: 'auto',
@@ -446,20 +461,32 @@ jQuery(document).ready(function($) {
 		placeholder: <?= $data['show_prev'] ? json_encode(_('value')) : '""' ?>,
 		value: <?= json_encode($data['prev_value']) ?>,
 		monospace_font: false,
-		maxlength: 65535,
 		disabled: <?= $data['show_prev'] ? 'false' : 'true' ?>,
 		grow: 'auto',
 		rows: 0
 	});
 
 	<?php if ($data['is_item_testable']): ?>
-		$('#get_value').on('change', function() {
-			$rows = $('#host_address_row, #proxy_hostid_row, #get_value_row'),
-			$form = $('#preprocessing-test-form'),
-			$submit_btn = overlays_stack.getById('item-test').$btn_submit;
+		$('#not_supported').on('change', function() {
+			var $form = $('#preprocessing-test-form');
 
 			if ($(this).is(':checked')) {
 				$('#value', $form).multilineInput('setReadOnly');
+			}
+			else {
+				$('#value', $form).multilineInput('unsetReadOnly');
+			}
+		});
+
+		$('#get_value').on('change', function() {
+			var $rows = $('.js-host-address-row, .js-proxy-hostid-row, .js-get-value-row, [class*=js-popup-row-snmp]'),
+				$form = $('#preprocessing-test-form'),
+				$submit_btn = overlays_stack.getById('item-test').$btn_submit,
+				$not_supported = $('#not_supported', $form);
+
+			if ($(this).is(':checked')) {
+				$('#value', $form).multilineInput('setReadOnly');
+				$not_supported.prop('disabled', true);
 
 				<?php if ($data['show_prev']): ?>
 					$('#prev_value', $form).multilineInput('setReadOnly');
@@ -480,9 +507,54 @@ jQuery(document).ready(function($) {
 
 				$submit_btn.html('<?= _('Get value and test') ?>');
 				$rows.show();
+
+				<?php if ($data['show_snmp_form']): ?>
+					$('#interface_details_version').on('change', function (e) {
+						$(`.js-popup-row-snmp-community, .js-popup-row-snmpv3-contextname,
+							.js-popup-row-snmpv3-securityname, .js-popup-row-snmpv3-securitylevel,
+							.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase,
+							.js-popup-row-snmpv3-privprotocol, .js-popup-row-snmpv3-privpassphrase`).hide();
+
+						switch (e.target.value) {
+							case '<?= SNMP_V1 ?>':
+								$('#interface_details_securitylevel').off('change');
+								$('.js-popup-row-snmp-community').show();
+								break;
+							case '<?= SNMP_V2C ?>':
+								$('#interface_details_securitylevel').off('change');
+								$('.js-popup-row-snmp-community').show();
+								break;
+							case '<?= SNMP_V3 ?>':
+								$(`.js-popup-row-snmpv3-contextname, .js-popup-row-snmpv3-securityname,
+									.js-popup-row-snmpv3-securitylevel`).show();
+
+								$('#interface_details_securitylevel').on('change', function (e) {
+									$(`.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase,
+										.js-popup-row-snmpv3-privprotocol, .js-popup-row-snmpv3-privpassphrase`).hide();
+									switch (e.target.value) {
+										case '<?= ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV ?>':
+											$(`.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase`)
+												.show();
+											break;
+										case '<?= ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV ?>':
+											$(`.js-popup-row-snmpv3-authprotocol, .js-popup-row-snmpv3-authpassphrase,
+												.js-popup-row-snmpv3-privprotocol, .js-popup-row-snmpv3-privpassphrase`)
+												.show();
+											break;
+									}
+
+									overlays_stack.end().centerDialog();
+								}).trigger('change');
+								break;
+						}
+
+						overlays_stack.end().centerDialog();
+					}).trigger('change');
+				<?php endif ?>
 			}
 			else {
-				$('#value', $form).multilineInput('unsetReadOnly');
+				!$not_supported.is(':checked') && $('#value', $form).multilineInput('unsetReadOnly');
+				$not_supported.prop('disabled', false);
 
 				<?php if ($data['show_prev']): ?>
 					$('#prev_value', $form).multilineInput('unsetReadOnly');
