@@ -46,7 +46,7 @@ $tags_form_grid = (new CFormGrid())
 					->setHeader([_('Name'), _('Value'), _('Action')])
 					->setId('tags-table'),
 				(new CScriptTemplate('tag-row-tmpl'))
-					->addItem(renderTagTableRow('#{rowNum}', '', '', ['add_post_js' => false]))
+					->addItem(renderTagTableRow('#{rowNum}', '', '', ZBX_TAG_MANUAL, ['add_post_js' => false]))
 			]))
 				->setId('tags-div')
 				->addClass(ZBX_STYLE_TABLE_FORMS)
@@ -57,6 +57,7 @@ $form->addItem($tags_form_grid);
 
 $output = [
 	'header' => $data['title'],
+	'doc_url' => CDocHelper::getUrl(CDocHelper::POPUP_MASSUPDATE_SERVICE),
 	'body' => $form->toString(),
 	'buttons' => [
 		[
