@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -107,13 +107,19 @@ class CHttpTestManager {
 
 		foreach ($httpTests as $hnum => &$httpTest) {
 			$httpTest['httptestid'] = $httpTestIds[$hnum];
+			$itemids = [];
 
-			$this->createHttpTestItems($httpTest);
-			$this->createStepsReal($httpTest, $httpTest['steps']);
+			$this->createHttpTestItems($httpTest, $itemids);
+			$this->createStepsReal($httpTest, $httpTest['steps'], $itemids);
+
+			if (array_key_exists('tags', $httpTest) && $httpTest['tags']) {
+				self::createItemsTags($httpTest['tags'], $itemids);
+			}
 		}
 		unset($httpTest);
 
 		$this->updateHttpTestFields($httpTests, 'create');
+		self::createHttpTestTags($httpTests);
 
 		return $httpTests;
 	}
@@ -126,24 +132,23 @@ class CHttpTestManager {
 	 * @return array
 	 */
 	public function update(array $httptests) {
-		$httptestids = zbx_objectValues($httptests, 'httptestid');
-
 		$db_httptests = API::HttpTest()->get([
-			'output' => ['httptestid', 'name', 'applicationid', 'delay', 'status', 'agent', 'authentication',
+			'output' => ['httptestid', 'name', 'delay', 'status', 'agent', 'authentication',
 				'http_user', 'http_password', 'hostid', 'templateid', 'http_proxy', 'retries', 'ssl_cert_file',
 				'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host'
 			],
-			'httptestids' => $httptestids,
 			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required', 'status_codes',
 				'follow_redirects', 'retrieve_mode'
 			],
-			'editable' => true,
+			'httptestids' => array_column($httptests, 'httptestid'),
+			'nopermissions' => true,
 			'preservekeys' => true
 		]);
 
 		$deleteStepItemIds = [];
 		$steps_create = [];
 		$steps_update = [];
+		$itemids = [];
 
 		foreach ($httptests as $key => $httptest) {
 			$db_httptest = $db_httptests[$httptest['httptestid']];
@@ -157,41 +162,48 @@ class CHttpTestManager {
 				'where' => ['httptestid' => $httptest['httptestid']]
 			]);
 
+			if (!array_key_exists($httptest['httptestid'], $itemids)) {
+				$itemids[$httptest['httptestid']] = [];
+			}
+
 			$checkItemsUpdate = [];
-			$updateFields = [];
-			$itemids = [];
 			$dbCheckItems = DBselect(
-				'SELECT i.itemid,hi.type'.
+				'SELECT i.itemid,i.name,i.key_,hi.type'.
 				' FROM items i,httptestitem hi'.
 				' WHERE hi.httptestid='.zbx_dbstr($httptest['httptestid']).
 					' AND hi.itemid=i.itemid'
 			);
 			while ($checkitem = DBfetch($dbCheckItems)) {
-				$itemids[] = $checkitem['itemid'];
+				$itemids[$httptest['httptestid']][] = $checkitem['itemid'];
+				$update_fields = [];
 
-				if (isset($httptest['name'])) {
-					$updateFields['key_'] = $this->getTestKey($checkitem['type'], $httptest['name']);
+				$update_fields['name'] = $this->getTestName($checkitem['type'], $httptest['name']);
+				if ($update_fields['name'] === $checkitem['name']) {
+					unset($update_fields['name']);
+				}
+
+				$update_fields['key_'] = $this->getTestKey($checkitem['type'], $httptest['name']);
+				if ($update_fields['key_'] === $checkitem['key_']) {
+					unset($update_fields['key_']);
 				}
 
 				if (isset($httptest['status'])) {
-					$updateFields['status'] = (HTTPTEST_STATUS_ACTIVE == $httptest['status']) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
+					$update_fields['status'] = (HTTPTEST_STATUS_ACTIVE == $httptest['status'])
+						? ITEM_STATUS_ACTIVE
+						: ITEM_STATUS_DISABLED;
 				}
 				if (isset($httptest['delay'])) {
-					$updateFields['delay'] = $httptest['delay'];
+					$update_fields['delay'] = $httptest['delay'];
 				}
-				if (!empty($updateFields)) {
+				if (!empty($update_fields)) {
 					$checkItemsUpdate[] = [
-						'values' => $updateFields,
+						'values' => $update_fields,
 						'where' => ['itemid' => $checkitem['itemid']]
 					];
 				}
 			}
 			DB::update('items', $checkItemsUpdate);
 
-			if (isset($httptest['applicationid'])) {
-				$this->updateItemsApplications($itemids, $httptest['applicationid']);
-			}
-
 			if (array_key_exists('steps', $httptest)) {
 				$dbSteps = zbx_toHash($db_httptest['steps'], 'httpstepid');
 
@@ -218,14 +230,6 @@ class CHttpTestManager {
 
 					DB::delete('httpstep', ['httpstepid' => $stepidsDelete]);
 				}
-
-				// IF application ID was not set, use the ID from DB so new items can be linked.
-				if (!array_key_exists('applicationid', $httptest)) {
-					$httptest['applicationid'] = $db_httptest['applicationid'];
-				}
-				elseif (bccomp($httptest['applicationid'], $db_httptest['applicationid'])) {
-					unset($httptest['applicationid']);
-				}
 			}
 		}
 
@@ -237,44 +241,25 @@ class CHttpTestManager {
 		foreach ($httptests as $key => $httptest) {
 			if (array_key_exists('steps', $httptest)) {
 				if (array_key_exists($key, $steps_update)) {
-					$this->updateStepsReal($httptest, $steps_update[$key]);
+					$this->updateStepsReal($httptest, $steps_update[$key], $itemids[$httptest['httptestid']]);
 				}
 
 				if (array_key_exists($key, $steps_create)) {
-					$this->createStepsReal($httptest, $steps_create[$key]);
+					$this->createStepsReal($httptest, $steps_create[$key], $itemids[$httptest['httptestid']]);
 				}
 			}
 			else {
-				if (isset($httptest['applicationid'])) {
-					$dbStepIds = DBfetchColumn(DBselect(
-						'SELECT i.itemid'.
-						' FROM items i'.
-							' INNER JOIN httpstepitem hi ON hi.itemid=i.itemid'.
-						' WHERE '.dbConditionInt('hi.httpstepid', zbx_objectValues($db_httptests[$httptest['httptestid']]['steps'], 'httpstepid')))
-						, 'itemid'
-					);
-					$this->updateItemsApplications($dbStepIds, $httptest['applicationid']);
-				}
-
-				if (isset($httptest['status'])) {
-					$status = ($httptest['status'] == HTTPTEST_STATUS_ACTIVE) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
-
-					$itemIds = DBfetchColumn(DBselect(
-						'SELECT hsi.itemid'.
-							' FROM httpstep hs,httpstepitem hsi'.
-							' WHERE hs.httpstepid=hsi.httpstepid'.
-								' AND hs.httptestid='.zbx_dbstr($httptest['httptestid'])
-					), 'itemid');
-
-					DB::update('items', [
-						'values' => ['status' => $status],
-						'where' => ['itemid' => $itemIds]
-					]);
-				}
+				$this->updateStepItems($httptest, $db_httptests[$httptest['httptestid']]);
 			}
 		}
 
 		$this->updateHttpTestFields($httptests, 'update');
+		self::updateHttpTestTags($httptests);
+
+		foreach ($httptests as $httptest) {
+			$tags = array_key_exists('tags', $httptest) ? $httptest['tags'] : [];
+			self::updateItemsTags($tags, $itemids[$httptest['httptestid']]);
+		}
 
 		return $httptests;
 	}
@@ -289,7 +274,7 @@ class CHttpTestManager {
 		$hostIds = zbx_toArray($hostIds);
 
 		$httpTests = API::HttpTest()->get([
-			'output' => ['httptestid', 'name', 'applicationid', 'delay', 'status', 'agent', 'authentication',
+			'output' => ['httptestid', 'name', 'delay', 'status', 'agent', 'authentication',
 				'http_user', 'http_password', 'hostid', 'templateid', 'http_proxy', 'retries', 'ssl_cert_file',
 				'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'variables', 'headers'
 			],
@@ -297,7 +282,9 @@ class CHttpTestManager {
 			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required', 'status_codes',
 				'follow_redirects', 'retrieve_mode', 'variables', 'headers', 'query_fields'
 			],
-			'preservekeys' => true
+			'selectTags' => ['tag', 'value'],
+			'preservekeys' => true,
+			'nopermissions' => true
 		]);
 
 		$this->inherit($httpTests, $hostIds);
@@ -411,6 +398,7 @@ class CHttpTestManager {
 				}
 
 				$newHttpTest = $httpTest;
+				$newHttpTest['uuid'] = '';
 				$newHttpTest['hostid'] = $hostId;
 				$newHttpTest['templateid'] = $httpTestId;
 				if ($exHttpTest) {
@@ -426,10 +414,6 @@ class CHttpTestManager {
 					unset($newHttpTest['httptestid']);
 				}
 
-				if (!empty($newHttpTest['applicationid'])) {
-					$newHttpTest['applicationid'] = $this->findChildApplication($newHttpTest['applicationid'], $hostId);
-				}
-
 				$result[] = $newHttpTest;
 			}
 		}
@@ -449,8 +433,7 @@ class CHttpTestManager {
 		return ($http_test['http_proxy'] === $ex_http_test['http_proxy']
 				&& $http_test['agent'] === $ex_http_test['agent']
 				&& $http_test['retries'] == $ex_http_test['retries']
-				&& $http_test['delay'] === $ex_http_test['delay']
-				&& bccomp($http_test['applicationid'], $ex_http_test['applicationid']) == 0);
+				&& $http_test['delay'] === $ex_http_test['delay']);
 	}
 
 	/**
@@ -462,7 +445,10 @@ class CHttpTestManager {
 	 */
 	protected function createLinkageBetweenHttpTests($parentId, $childId) {
 		DB::update('httptest', [
-			'values' => ['templateid' => $parentId],
+			'values' => [
+				'templateid' => $parentId,
+				'uuid' => ''
+			],
 			'where' => ['httptestid' => $childId]
 		]);
 
@@ -501,26 +487,6 @@ class CHttpTestManager {
 		}
 	}
 
-	/**
-	 * Find application with same name on given host.
-	 *
-	 * @param $parentAppId
-	 * @param $childHostId
-	 *
-	 * @return string
-	 */
-	protected function findChildApplication($parentAppId, $childHostId) {
-		$childAppId = DBfetch(DBselect(
-			'SELECT a2.applicationid'.
-			' FROM applications a1'.
-				' INNER JOIN applications a2 ON a1.name=a2.name'.
-			' WHERE a1.applicationid='.zbx_dbstr($parentAppId).
-				' AND a2.hostid='.zbx_dbstr($childHostId))
-		);
-
-		return $childAppId['applicationid'];
-	}
-
 	/**
 	 * Find and set first parent id for http test.
 	 *
@@ -557,8 +523,7 @@ class CHttpTestManager {
 		}
 
 		$dbCursor = DBselect(
-			'SELECT ht.httptestid,ht.name,ht.applicationid,ht.delay,ht.agent,ht.hostid,ht.templateid,ht.http_proxy,'.
-				'ht.retries'.
+			'SELECT ht.httptestid,ht.name,ht.delay,ht.agent,ht.hostid,ht.templateid,ht.http_proxy,ht.retries'.
 			' FROM httptest ht'.
 			' WHERE '.dbConditionInt('ht.hostid', $hostIds)
 		);
@@ -623,7 +588,7 @@ class CHttpTestManager {
 			}
 
 			/*
-			 * Unset $http_tests and (later) put it back with actual httptestid as a key right after creating/updateing
+			 * Unset $http_tests and (later) put it back with actual httptestid as a key right after creating/updating
 			 * it. This is done in such a way because $http_tests array holds items with incremental keys which are not
 			 * a real httptestids.
 			 */
@@ -691,27 +656,28 @@ class CHttpTestManager {
 	 * Create items required for web scenario.
 	 *
 	 * @param array $http_test
+	 * @param array  $_itemids
 	 *
 	 * @throws Exception
 	 */
-	protected function createHttpTestItems(array $http_test) {
+	protected function createHttpTestItems(array $http_test, array &$_itemids): void {
 		$checkitems = [
 			[
-				'name'				=> 'Download speed for scenario "$1".',
+				'name'				=> $this->getTestName(HTTPSTEP_ITEM_TYPE_IN, $http_test['name']),
 				'key_'				=> $this->getTestKey(HTTPSTEP_ITEM_TYPE_IN, $http_test['name']),
 				'value_type'		=> ITEM_VALUE_TYPE_FLOAT,
 				'units'				=> 'Bps',
 				'httptestitemtype'	=> HTTPSTEP_ITEM_TYPE_IN
 			],
 			[
-				'name'				=> 'Failed step of scenario "$1".',
+				'name'				=> $this->getTestName(HTTPSTEP_ITEM_TYPE_LASTSTEP, $http_test['name']),
 				'key_'				=> $this->getTestKey(HTTPSTEP_ITEM_TYPE_LASTSTEP, $http_test['name']),
 				'value_type'		=> ITEM_VALUE_TYPE_UINT64,
 				'units'				=> '',
 				'httptestitemtype'	=> HTTPSTEP_ITEM_TYPE_LASTSTEP
 			],
 			[
-				'name'				=> 'Last error message of scenario "$1".',
+				'name'				=> $this->getTestName(HTTPSTEP_ITEM_TYPE_LASTERROR, $http_test['name']),
 				'key_'				=> $this->getTestKey(HTTPSTEP_ITEM_TYPE_LASTERROR, $http_test['name']),
 				'value_type'		=> ITEM_VALUE_TYPE_STR,
 				'units'				=> '',
@@ -752,12 +718,9 @@ class CHttpTestManager {
 		}
 		$itemids = DB::insert('items', $ins_items);
 
-		if (array_key_exists('applicationid', $http_test)) {
-			$this->createItemsApplications($itemids, $http_test['applicationid']);
-		}
-
 		$ins_item_rtdata = [];
 		foreach ($itemids as $itemid) {
+			$_itemids[] = $itemid;
 			$ins_item_rtdata[] = ['itemid' => $itemid];
 		}
 		DB::insertBatch('item_rtdata', $ins_item_rtdata, false);
@@ -968,12 +931,13 @@ class CHttpTestManager {
 	/**
 	 * Create web scenario steps with items.
 	 *
-	 * @param $http_test
-	 * @param $websteps
+	 * @param array  $http_test
+	 * @param array  $websteps
+	 * @param array  $_itemids
 	 *
 	 * @throws Exception
 	 */
-	protected function createStepsReal($http_test, $websteps) {
+	protected function createStepsReal(array $http_test, array $websteps, array &$_itemids): void {
 		foreach ($websteps as &$webstep) {
 			$webstep['httptestid'] = $http_test['httptestid'];
 
@@ -1013,21 +977,21 @@ class CHttpTestManager {
 
 			$stepitems = [
 				[
-					'name' => 'Download speed for step "$2" of scenario "$1".',
+					'name' => $this->getStepName(HTTPSTEP_ITEM_TYPE_IN, $http_test['name'], $webstep['name']),
 					'key_' => $this->getStepKey(HTTPSTEP_ITEM_TYPE_IN, $http_test['name'], $webstep['name']),
 					'value_type' => ITEM_VALUE_TYPE_FLOAT,
 					'units' => 'Bps',
 					'httpstepitemtype' => HTTPSTEP_ITEM_TYPE_IN
 				],
 				[
-					'name' => 'Response time for step "$2" of scenario "$1".',
+					'name' => $this->getStepName(HTTPSTEP_ITEM_TYPE_TIME, $http_test['name'], $webstep['name']),
 					'key_' => $this->getStepKey(HTTPSTEP_ITEM_TYPE_TIME, $http_test['name'], $webstep['name']),
 					'value_type' => ITEM_VALUE_TYPE_FLOAT,
 					'units' => 's',
 					'httpstepitemtype' => HTTPSTEP_ITEM_TYPE_TIME
 				],
 				[
-					'name' => 'Response code for step "$2" of scenario "$1".',
+					'name' => $this->getStepName(HTTPSTEP_ITEM_TYPE_RSPCODE, $http_test['name'], $webstep['name']),
 					'key_' => $this->getStepKey(HTTPSTEP_ITEM_TYPE_RSPCODE, $http_test['name'], $webstep['name']),
 					'value_type' => ITEM_VALUE_TYPE_UINT64,
 					'units' => '',
@@ -1066,11 +1030,8 @@ class CHttpTestManager {
 			}
 			$itemids = DB::insert('items', $ins_items);
 
-			if (array_key_exists('applicationid', $http_test)) {
-				$this->createItemsApplications($itemids, $http_test['applicationid']);
-			}
-
 			foreach ($stepitems as $inum => $item) {
+				$_itemids[] = $itemids[$inum];
 				$ins_httpstepitem[] = [
 					'httpstepid' => $webstep['httpstepid'],
 					'itemid' => $itemids[$inum],
@@ -1093,24 +1054,15 @@ class CHttpTestManager {
 	/**
 	 * Update web scenario steps.
 	 *
-	 * @param $httpTest
-	 * @param $websteps
+	 * @param array  $httpTest
+	 * @param array  $websteps
+	 * @param array  $itemids_per_http_tests
 	 *
 	 * @throws Exception
 	 */
-	protected function updateStepsReal($httpTest, $websteps) {
+	protected function updateStepsReal(array $httpTest, array $websteps, array &$_itemids): void {
 		$item_key_parser = new CItemKey();
 
-		// get all used keys
-		$webstepids = zbx_objectValues($websteps, 'httpstepid');
-		$dbKeys = DBfetchArrayAssoc(DBselect(
-			'SELECT i.key_'.
-			' FROM items i,httpstepitem hi'.
-			' WHERE '.dbConditionInt('hi.httpstepid', $webstepids).
-				' AND hi.itemid=i.itemid')
-			, 'key_'
-		);
-
 		foreach ($websteps as &$webstep) {
 			if (array_key_exists('posts', $webstep)) {
 				if (is_array($webstep['posts'])) {
@@ -1133,16 +1085,16 @@ class CHttpTestManager {
 			]);
 
 			// update item keys
-			$itemids = [];
-			$stepitemsUpdate = $updateFields = [];
+			$stepitems_update = [];
 			$dbStepItems = DBselect(
-				'SELECT i.itemid,i.key_,hi.type'.
+				'SELECT i.itemid,i.name,i.key_,hi.type'.
 				' FROM items i,httpstepitem hi'.
 				' WHERE hi.httpstepid='.zbx_dbstr($webstep['httpstepid']).
 					' AND hi.itemid=i.itemid'
 			);
 			while ($stepitem = DBfetch($dbStepItems)) {
-				$itemids[] = $stepitem['itemid'];
+				$_itemids[] = $stepitem['itemid'];
+				$update_fields = [];
 
 				if (isset($httpTest['name']) || isset($webstep['name'])) {
 					if (!isset($httpTest['name']) || !isset($webstep['name'])) {
@@ -1155,31 +1107,34 @@ class CHttpTestManager {
 						}
 					}
 
-					$updateFields['key_'] = $this->getStepKey($stepitem['type'], $httpTest['name'], $webstep['name']);
-				}
-				if (isset($dbKeys[$updateFields['key_']])) {
-					unset($updateFields['key_']);
+					$update_fields['name'] = $this->getStepName($stepitem['type'], $httpTest['name'], $webstep['name']);
+					if ($update_fields['name'] === $stepitem['name']) {
+						unset($update_fields['name']);
+					}
+
+					$update_fields['key_'] = $this->getStepKey($stepitem['type'], $httpTest['name'], $webstep['name']);
+					if ($update_fields['key_'] === $stepitem['key_']) {
+						unset($update_fields['key_']);
+					}
 				}
 				if (isset($httpTest['status'])) {
-					$updateFields['status'] = (HTTPTEST_STATUS_ACTIVE == $httpTest['status']) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
+					$update_fields['status'] = (HTTPTEST_STATUS_ACTIVE == $httpTest['status'])
+						? ITEM_STATUS_ACTIVE
+						: ITEM_STATUS_DISABLED;
 				}
 				if (isset($httpTest['delay'])) {
-					$updateFields['delay'] = $httpTest['delay'];
+					$update_fields['delay'] = $httpTest['delay'];
 				}
-				if (!empty($updateFields)) {
-					$stepitemsUpdate[] = [
-						'values' => $updateFields,
+				if (!empty($update_fields)) {
+					$stepitems_update[] = [
+						'values' => $update_fields,
 						'where' => ['itemid' => $stepitem['itemid']]
 					];
 				}
 			}
 
-			if ($stepitemsUpdate) {
-				DB::update('items', $stepitemsUpdate);
-			}
-
-			if (array_key_exists('applicationid', $httpTest)) {
-				$this->updateItemsApplications($itemids, $httpTest['applicationid']);
+			if ($stepitems_update) {
+				DB::update('items', $stepitems_update);
 			}
 		}
 
@@ -1187,47 +1142,138 @@ class CHttpTestManager {
 	}
 
 	/**
-	 * Update web item application linkage.
+	 * Update web items after changes in web scenario.
+	 * This should be used, when individual steps are not being updated.
 	 *
-	 * @param array  $itemids
-	 * @param string $applicationid
+	 * @param array $httptest
+	 * @param array $db_httptest
 	 */
-	protected function updateItemsApplications(array $itemids, $applicationid) {
-		if ($applicationid == 0) {
-			DB::delete('items_applications', ['itemid' => $itemids]);
-		}
-		else {
-			$linked_itemids = DBfetchColumn(
-				DBselect('SELECT ia.itemid FROM items_applications ia WHERE '.dbConditionInt('ia.itemid', $itemids)),
-				'itemid'
-			);
+	protected function updateStepItems(array $httptest, array $db_httptest): void {
+		$has_status = array_key_exists('status', $httptest);
+		$has_name_changed = ($httptest['name'] !== $db_httptest['name']);
 
-			if ($linked_itemids) {
-				DB::update('items_applications', [
-					'values' => ['applicationid' => $applicationid],
-					'where' => ['itemid' => $linked_itemids]
-				]);
+		$stepids = array_column($db_httptest['steps'], 'httpstepid');
+
+		$stepitems = DBfetchArrayAssoc(DBselect(
+			'SELECT i.itemid, hsi.httpstepid, hsi.type'.
+				' FROM items i,httpstepitem hsi'.
+				' WHERE i.itemid=hsi.itemid'.
+					' AND '.dbConditionInt('hsi.httpstepid', $stepids)
+		), 'itemid');
+
+		$stepitemids = array_keys($stepitems);
+
+		if ($has_status) {
+			$status = ($httptest['status'] == HTTPTEST_STATUS_ACTIVE)
+				? ITEM_STATUS_ACTIVE
+				: ITEM_STATUS_DISABLED;
+
+			DB::update('items', [
+				'values' => ['status' => $status],
+				'where' => ['itemid' => $stepitemids]
+			]);
+		}
+
+		if ($has_name_changed) {
+			$db_websteps = zbx_toHash($db_httptest['steps'], 'httpstepid');
+			$stepitems_update = [];
+
+			foreach ($stepitems as $stepitem) {
+				$db_webstep = $db_websteps[$stepitem['httpstepid']];
+
+				$stepitems_update[] = [
+					'values' => [
+						'name' => $this->getStepName($stepitem['type'], $httptest['name'], $db_webstep['name']),
+						'key_' => $this->getStepKey($stepitem['type'], $httptest['name'], $db_webstep['name'])
+					],
+					'where' => ['itemid' => $stepitem['itemid']]
+				];
 			}
 
-			$this->createItemsApplications(array_diff($itemids, $linked_itemids), $applicationid);
+			if ($stepitems_update) {
+				DB::update('items', $stepitems_update);
+			}
 		}
 	}
 
 	/**
-	 * Create web item application linkage.
+	 * Create tags for http test and http test step items.
+	 * All items should belong to the same http test and should have same set of tags.
 	 *
-	 * @param array  $itemids
-	 * @param string $applicationid
+	 * @static
+	 *
+	 * @param array $tags     New tags to save.
+	 * @param array $itemids  List of itemids.
 	 */
-	protected function createItemsApplications(array $itemids, $applicationid) {
-		if ($applicationid != 0 && $itemids) {
-			$insert = [];
-
+	protected static function createItemsTags(array $tags, array $itemids): void {
+		$new_tags = [];
+		foreach ($tags as $tag) {
 			foreach ($itemids as $itemid) {
-				$insert[] = ['itemid' => $itemid, 'applicationid' => $applicationid];
+				$new_tags[] = $tag + ['itemid' => $itemid];
 			}
+		}
 
-			DB::insertBatch('items_applications', $insert);
+		DB::insert('item_tag', $new_tags);
+	}
+
+	/**
+	 * Update step item tags.
+	 * Function assumes that all step items has same set of tags. Not suitable for steps from different web scenarios.
+	 *
+	 * @static
+	 *
+	 * @param array $tags         New tags to save.
+	 * @param array $stepitemids  List of step itemids to update.
+	 */
+	protected static function updateItemsTags(array $tags, array $stepitemids): void {
+		// Select tags from database.
+		$db_tags_raw = DB::select('item_tag', [
+			'output' => ['itemtagid', 'tag', 'value', 'itemid'],
+			'filter' => ['itemid' => $stepitemids]
+		]);
+		$db_tags = [];
+		foreach ($db_tags_raw as $tag) {
+			$db_tags[$tag['itemid']][$tag['tag']][$tag['value']] = $tag['itemtagid'];
+		}
+
+		// Make array with new tags.
+		$item_tags_add = array_fill_keys($stepitemids, $tags);
+
+		// Unset tags which don't need to add/delete.
+		foreach ($db_tags as $stepitemid => $item_tags_del) {
+			foreach ($item_tags_add[$stepitemid] as $new_tag_key => $tag_add) {
+				if (array_key_exists($tag_add['tag'], $item_tags_del)
+						&& array_key_exists($tag_add['value'], $item_tags_del[$tag_add['tag']])) {
+					unset($item_tags_add[$stepitemid][$new_tag_key],
+						$db_tags[$stepitemid][$tag_add['tag']][$tag_add['value']]
+					);
+				}
+			}
+		}
+
+		// Delete tags.
+		$del_tagids = [];
+		foreach ($db_tags as $db_tag) {
+			foreach ($db_tag as $db_tagids) {
+				if ($db_tagids) {
+					$del_tagids = array_merge($del_tagids, array_values($db_tagids));
+				}
+			}
+		}
+		if ($del_tagids) {
+			DB::delete('item_tag', ['itemtagid' => $del_tagids]);
+		}
+
+		// Add new tags.
+		$new_tags = [];
+		foreach ($item_tags_add as $stepitemid => $tags) {
+			foreach ($tags as $tag) {
+				$tag['itemid'] = $stepitemid;
+				$new_tags[] = $tag;
+			}
+		}
+		if ($new_tags) {
+			DB::insert('item_tag', $new_tags);
 		}
 	}
 
@@ -1235,43 +1281,86 @@ class CHttpTestManager {
 	 * Get item key for test item.
 	 *
 	 * @param int    $type
-	 * @param string $testName
+	 * @param string $test_name
 	 *
-	 * @return bool|string
+	 * @return string
 	 */
-	protected function getTestKey($type, $testName) {
+	protected function getTestKey(int $type, string $test_name): string {
 		switch ($type) {
 			case HTTPSTEP_ITEM_TYPE_IN:
-				return 'web.test.in['.quoteItemKeyParam($testName).',,bps]';
+				return 'web.test.in['.quoteItemKeyParam($test_name).',,bps]';
 			case HTTPSTEP_ITEM_TYPE_LASTSTEP:
-				return 'web.test.fail['.quoteItemKeyParam($testName).']';
+				return 'web.test.fail['.quoteItemKeyParam($test_name).']';
 			case HTTPSTEP_ITEM_TYPE_LASTERROR:
-				return 'web.test.error['.quoteItemKeyParam($testName).']';
+				return 'web.test.error['.quoteItemKeyParam($test_name).']';
 		}
 
-		return false;
+		return 'unknown';
+	}
+
+	/**
+	 * Get item name for test item.
+	 *
+	 * @param int    $type
+	 * @param string $test_name
+	 *
+	 * @return string
+	 */
+	protected function getTestName(int $type, string $test_name): string {
+		switch ($type) {
+			case HTTPSTEP_ITEM_TYPE_IN:
+				return 'Download speed for scenario "'.$test_name.'".';
+			case HTTPSTEP_ITEM_TYPE_LASTSTEP:
+				return 'Failed step of scenario "'.$test_name.'".';
+			case HTTPSTEP_ITEM_TYPE_LASTERROR:
+				return 'Last error message of scenario "'.$test_name.'".';
+		}
+
+		return 'unknown';
 	}
 
 	/**
 	 * Get item key for step item.
 	 *
 	 * @param int    $type
-	 * @param string $testName
-	 * @param string $stepName
+	 * @param string $test_name
+	 * @param string $step_name
 	 *
-	 * @return bool|string
+	 * @return string
 	 */
-	protected function getStepKey($type, $testName, $stepName) {
+	protected function getStepKey(int $type, string $test_name, string $step_name): string {
 		switch ($type) {
 			case HTTPSTEP_ITEM_TYPE_IN:
-				return 'web.test.in['.quoteItemKeyParam($testName).','.quoteItemKeyParam($stepName).',bps]';
+				return 'web.test.in['.quoteItemKeyParam($test_name).','.quoteItemKeyParam($step_name).',bps]';
 			case HTTPSTEP_ITEM_TYPE_TIME:
-				return 'web.test.time['.quoteItemKeyParam($testName).','.quoteItemKeyParam($stepName).',resp]';
+				return 'web.test.time['.quoteItemKeyParam($test_name).','.quoteItemKeyParam($step_name).',resp]';
 			case HTTPSTEP_ITEM_TYPE_RSPCODE:
-				return 'web.test.rspcode['.quoteItemKeyParam($testName).','.quoteItemKeyParam($stepName).']';
+				return 'web.test.rspcode['.quoteItemKeyParam($test_name).','.quoteItemKeyParam($step_name).']';
 		}
 
-		return false;
+		return 'unknown';
+	}
+
+	/**
+	 * Get item name for step item.
+	 *
+	 * @param int    $type
+	 * @param string $test_name
+	 * @param string $step_name
+	 *
+	 * @return string
+	 */
+	protected function getStepName(int $type, string $test_name, string $step_name): string {
+		switch ($type) {
+			case HTTPSTEP_ITEM_TYPE_IN:
+				return 'Download speed for step "'.$step_name.'" of scenario "'.$test_name.'".';
+			case HTTPSTEP_ITEM_TYPE_TIME:
+				return 'Response time for step "'.$step_name.'" of scenario "'.$test_name.'".';
+			case HTTPSTEP_ITEM_TYPE_RSPCODE:
+				return 'Response code for step "'.$step_name.'" of scenario "'.$test_name.'".';
+		}
+
+		return 'unknown';
 	}
 
 	/**
@@ -1282,7 +1371,7 @@ class CHttpTestManager {
 	 * - lastfailedstep - number of the last failed step
 	 * - error          - error message
 	 *
-	 * If a HTTP test has not been executed in last ZBX_HISTORY_PERIOD, no value will be returned.
+	 * If a HTTP test has not been executed in last CSettingsHelper::HISTORY_PERIOD, no value will be returned.
 	 *
 	 * @param array $httpTestIds
 	 *
@@ -1297,7 +1386,9 @@ class CHttpTestManager {
 				' AND '.dbConditionInt('hti.httptestid', $httpTestIds)
 		));
 
-		$history = Manager::History()->getLastValues($httpItems, 1, ZBX_HISTORY_PERIOD);
+		$history = Manager::History()->getLastValues($httpItems, 1, timeUnitToSeconds(CSettingsHelper::get(
+			CSettingsHelper::HISTORY_PERIOD
+		)));
 
 		$data = [];
 
@@ -1325,4 +1416,98 @@ class CHttpTestManager {
 
 		return $data;
 	}
+
+	/**
+	 * Record web scenario tags into database.
+	 *
+	 * @static
+	 *
+	 * @param array  $http_tests
+	 * @param array  $http_tests[]['tags']
+	 * @param string $http_tests[]['tags'][]['tag']
+	 * @param string $http_tests[]['tags'][]['value']
+	 * @param string $http_tests[]['httptestid']
+	 */
+	protected static function createHttpTestTags(array $http_tests): void {
+		$new_tags = [];
+		foreach ($http_tests as $http_test) {
+			if (!array_key_exists('tags', $http_test)) {
+				continue;
+			}
+
+			foreach ($http_test['tags'] as $tag) {
+				$new_tags[] = $tag + ['httptestid' => $http_test['httptestid']];
+			}
+		}
+
+		if ($new_tags) {
+			DB::insert('httptest_tag', $new_tags);
+		}
+	}
+
+	/**
+	 * Update web scenario tags.
+	 *
+	 * @static
+	 *
+	 * @param array  $http_tests
+	 * @param string $http_tests[]['httptestid']
+	 * @param array  $http_tests[]['tags']
+	 * @param string $http_tests[]['tags'][]['tag']
+	 * @param string $http_tests[]['tags'][]['value']
+	 */
+	protected static function updateHttpTestTags(array $http_tests): void {
+		$http_tests = zbx_toHash($http_tests, 'httptestid');
+
+		// Select tags from database.
+		$db_httptest_tags_raw = DB::select('httptest_tag', [
+			'output' => ['httptesttagid', 'httptestid', 'tag', 'value'],
+			'filter' => ['httptestid' => array_column($http_tests, 'httptestid')]
+		]);
+
+		$db_httptest_tags = [];
+		foreach ($db_httptest_tags_raw as $tag) {
+			$db_httptest_tags[$tag['httptestid']][] = $tag;
+		}
+
+		// Find which tags must be added/deleted.
+		$del_tagids = [];
+		foreach ($db_httptest_tags as $httptestid => $db_tags) {
+			if (!array_key_exists('tags', $http_tests[$httptestid])) {
+				continue;
+			}
+
+			foreach ($db_tags as $del_tag_key => $tag_delete) {
+				foreach ($http_tests[$httptestid]['tags'] as $new_tag_key => $tag_add) {
+					if ($tag_delete['tag'] === $tag_add['tag'] && $tag_delete['value'] === $tag_add['value']) {
+						unset($db_tags[$del_tag_key], $http_tests[$httptestid]['tags'][$new_tag_key]);
+						continue 2;
+					}
+				}
+			}
+
+			if ($db_tags) {
+				$del_tagids = array_merge($del_tagids, array_column($db_tags, 'httptesttagid'));
+			}
+		}
+
+		$new_tags = [];
+		foreach ($http_tests as $httptestid => $http_test) {
+			if (!array_key_exists('tags', $http_test)) {
+				continue;
+			}
+
+			foreach ($http_test['tags'] as $tag) {
+				$tag['httptestid'] = $httptestid;
+				$new_tags[] = $tag;
+			}
+		}
+
+		if ($del_tagids) {
+			DB::delete('httptest_tag', ['httptesttagid' => $del_tagids]);
+		}
+		if ($new_tags) {
+			DB::insert('httptest_tag', $new_tags);
+		}
+	}
 }
