@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -242,6 +242,12 @@ class CItemManager {
 			DB::insertBatch('housekeeper', $ins_housekeeper);
 		}
 
+		DB::delete('item_tag', ['itemid' => $del_itemids]);
+		DB::delete('item_preproc', ['itemid' => $del_itemids]);
+		DB::update('items', [
+			'values' => ['templateid' => 0, 'master_itemid' => 0],
+			'where' => ['itemid' => $del_itemids]
+		]);
 		DB::delete('items', ['itemid' => $del_itemids]);
 	}
 }
