@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,12 @@
  */
 class CAlert extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => [
+			'min_user_type' => USER_TYPE_ZABBIX_USER
+		]
+	];
+
 	protected $tableName = 'alerts';
 	protected $tableAlias = 'a';
 	protected $sortColumns = ['alertid', 'clock', 'eventid', 'status', 'sendto', 'mediatypeid'];
@@ -36,7 +42,6 @@ class CAlert extends CApiService {
 	 * @param array $options['hostids']
 	 * @param array $options['groupids']
 	 * @param array $options['alertids']
-	 * @param array $options['applicationids']
 	 * @param array $options['status']
 	 * @param bool  $options['editable']
 	 * @param array $options['extendoutput']
@@ -85,6 +90,7 @@ class CAlert extends CApiService {
 			'selectUsers'				=> null,
 			'selectHosts'				=> null,
 			'countOutput'				=> false,
+			'groupCount'				=> false,
 			'preservekeys'				=> false,
 			'editable'					=> false,
 			'sortfield'					=> '',
@@ -140,7 +146,7 @@ class CAlert extends CApiService {
 
 		// Oracle does not support using distinct with nclob fields, so we must use exists instead of joins
 		if ($options['eventsource'] == EVENT_SOURCE_TRIGGERS
-				|| $options['eventsource'] == EVENT_SOURCE_AUTO_REGISTRATION) {
+				|| $options['eventsource'] == EVENT_SOURCE_AUTOREGISTRATION) {
 			/*
 			 * Performance optimization: events with such sources does not have multiple objects therefore we can ignore
 			 * event object in SQL requests.
@@ -241,8 +247,8 @@ class CAlert extends CApiService {
 		}
 
 		// objectids
-		if ($options['objectids'] !== null
-				&& in_array($options['eventobject'], [EVENT_OBJECT_TRIGGER, EVENT_OBJECT_ITEM, EVENT_OBJECT_LLDRULE])) {
+		if ($options['objectids'] !== null && in_array($options['eventobject'],
+				[EVENT_OBJECT_TRIGGER, EVENT_OBJECT_ITEM, EVENT_OBJECT_LLDRULE, EVENT_OBJECT_SERVICE])) {
 			zbx_value2array($options['objectids']);
 
 			// Oracle does not support using distinct with nclob fields, so we must use exists instead of joins
@@ -259,6 +265,10 @@ class CAlert extends CApiService {
 			zbx_value2array($options['eventids']);
 
 			$sqlParts['where'][] = dbConditionInt('a.eventid', $options['eventids']);
+
+			if ($options['groupCount']) {
+				$sqlParts['group']['a'] = 'a.eventid';
+			}
 		}
 
 		// actionids
@@ -322,10 +332,15 @@ class CAlert extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($alert = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
-				$result = $alert['rowscount'];
+				if ($options['groupCount']) {
+					$result[] = $alert;
+				}
+				else {
+					$result = $alert['rowscount'];
+				}
 			}
 			else {
 				$result[$alert['alertid']] = $alert;
@@ -400,6 +415,9 @@ class CAlert extends CApiService {
 
 		// adding hosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] !== API_OUTPUT_COUNT) {
+			$hosts = [];
+			$relationMap = new CRelationMap();
+
 			// trigger events
 			if ($options['eventobject'] == EVENT_OBJECT_TRIGGER) {
 				$query = DBselect(
@@ -426,15 +444,20 @@ class CAlert extends CApiService {
 				);
 			}
 
-			$relationMap = new CRelationMap();
 			while ($relation = DBfetch($query)) {
 				$relationMap->addRelation($relation['alertid'], $relation['hostid']);
 			}
-			$hosts = API::Host()->get([
-				'output' => $options['selectHosts'],
-				'hostids' => $relationMap->getRelatedIds(),
-				'preservekeys' => true
-			]);
+
+			$related_ids = $relationMap->getRelatedIds();
+
+			if ($related_ids) {
+				$hosts = API::Host()->get([
+					'output' => $options['selectHosts'],
+					'hostids' => $related_ids,
+					'preservekeys' => true
+				]);
+			}
+
 			$result = $relationMap->mapMany($result, $hosts, 'hosts');
 		}
 
