@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@ class DB {
 	const FIELD_TYPE_BLOB = 'blob';
 	const FIELD_TYPE_TEXT = 'text';
 	const FIELD_TYPE_NCLOB = 'nclob';
+	const FIELD_TYPE_CUID = 'cuid';
 
 	private static $schema = null;
 
@@ -225,12 +226,12 @@ class DB {
 	 *
 	 * @static
 	 *
-	 * @param string $tableName
+	 * @param string $table_name
 	 *
-	 * @return string|array
+	 * @return string
 	 */
-	protected static function getPk($tableName) {
-		$schema = self::getSchema($tableName);
+	public static function getPk(string $table_name): string {
+		$schema = self::getSchema($table_name);
 
 		return $schema['key'];
 	}
@@ -381,6 +382,7 @@ class DB {
 			}
 			else {
 				switch ($tableSchema['fields'][$field]['type']) {
+					case self::FIELD_TYPE_CUID:
 					case self::FIELD_TYPE_CHAR:
 						$length = mb_strlen($values[$field]);
 
@@ -390,6 +392,7 @@ class DB {
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+
 					case self::FIELD_TYPE_ID:
 					case self::FIELD_TYPE_UINT:
 						if (!zbx_ctype_digit($values[$field])) {
@@ -397,18 +400,21 @@ class DB {
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+
 					case self::FIELD_TYPE_INT:
 						if (!zbx_is_int($values[$field])) {
 							self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for int field "%2$s".', $values[$field], $field));
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+
 					case self::FIELD_TYPE_FLOAT:
 						if (!is_numeric($values[$field])) {
 							self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for float field "%2$s".', $values[$field], $field));
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+
 					case self::FIELD_TYPE_TEXT:
 						if ($DB['TYPE'] == ZBX_DB_ORACLE) {
 							$length = mb_strlen($values[$field]);
@@ -420,6 +426,7 @@ class DB {
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+
 					case self::FIELD_TYPE_NCLOB:
 						// Using strlen because 4000 bytes is largest possible string literal in oracle query.
 						if ($DB['TYPE'] == ZBX_DB_ORACLE && strlen($values[$field]) > ORACLE_MAX_STRING_SIZE) {
@@ -430,6 +437,21 @@ class DB {
 							$values[$field] = zbx_dbstr($values[$field]);
 						}
 						break;
+
+					case self::FIELD_TYPE_BLOB:
+						switch ($DB['TYPE']) {
+							case ZBX_DB_MYSQL:
+								$values[$field] = zbx_dbstr($values[$field]);
+								break;
+
+							case ZBX_DB_POSTGRESQL:
+								$values[$field] = "'".pg_escape_bytea($DB['DB'], $values[$field])."'";
+								break;
+
+							case ZBX_DB_ORACLE:
+								// Do nothing; Check CImage.php to see how to update BLOB data with ORACLE DB.
+								break;
+						}
 				}
 			}
 		}
@@ -498,12 +520,7 @@ class DB {
 	 */
 	public static function insert($table, $values, $getids = true) {
 		$table_schema = self::getSchema($table);
-		$fields = [];
-
-		foreach ($values as $key => $row) {
-			$fields += array_diff_key($row, $fields);
-		}
-
+		$fields = array_reduce($values, 'array_merge', []);
 		$fields = array_intersect_key($fields, $table_schema['fields']);
 
 		foreach ($fields as $field => &$value) {
@@ -514,14 +531,7 @@ class DB {
 		unset($value);
 
 		foreach ($values as $key => &$row) {
-			$row += $fields;
-
-			$ordered_row = [];
-			foreach ($fields as $field => $foo) {
-				$ordered_row[$field] = $row[$field];
-			}
-
-			$row = $ordered_row;
+			$row = array_merge($fields, $row);
 		}
 		unset($row);
 
@@ -542,17 +552,62 @@ class DB {
 
 		$mandatory_fields = [];
 
-		if ($DB['TYPE'] == ZBX_DB_MYSQL) {
-			foreach ($table_schema['fields'] as $name => $field) {
-				if ($field['type'] == self::FIELD_TYPE_TEXT || $field['type'] == self::FIELD_TYPE_NCLOB) {
-					$mandatory_fields += [$name => $field['default']];
+		switch ($DB['TYPE']) {
+			case ZBX_DB_MYSQL:
+				foreach ($table_schema['fields'] as $name => $field) {
+					if ($field['type'] == self::FIELD_TYPE_TEXT || $field['type'] == self::FIELD_TYPE_NCLOB) {
+						$mandatory_fields += [$name => $field['default']];
+					}
+				}
+				break;
+
+			case ZBX_DB_ORACLE:
+				foreach ($table_schema['fields'] as $name => $field) {
+					if ($field['type'] == self::FIELD_TYPE_BLOB) {
+						$mandatory_fields += [$name => 'EMPTY_BLOB()'];
+					}
 				}
-			}
 		}
 
 		return $mandatory_fields;
 	}
 
+	/**
+	 * Add IDs to inserted rows.
+	 *
+	 * @param string $table
+	 * @param array  $values
+	 *
+	 * @return array An array of IDs with the keys preserved.
+	 */
+	private static function addIds(string $table, array &$values): array {
+		$table_schema = self::getSchema($table);
+		$resultids = [];
+
+		if ($table_schema['fields'][$table_schema['key']]['type'] === DB::FIELD_TYPE_ID) {
+			$id = self::reserveIds($table, count($values));
+		}
+
+		foreach ($values as $key => &$row) {
+			switch ($table_schema['fields'][$table_schema['key']]['type']) {
+				case DB::FIELD_TYPE_ID:
+					$resultids[$key] = $id;
+					$row = [$table_schema['key'] => $id] + $row;
+					$id = bcadd($id, 1, 0);
+					break;
+
+				case DB::FIELD_TYPE_CUID:
+					$id = CCuid::generate();
+					$resultids[$key] = $id;
+					$row = [$table_schema['key'] => $id] + $row;
+					break;
+			}
+		}
+		unset($row);
+
+		return $resultids;
+	}
+
 	/**
 	 * Insert batch data into DB.
 	 *
@@ -560,30 +615,22 @@ class DB {
 	 * @param array  $values pair of fieldname => fieldvalue
 	 * @param bool   $getids
 	 *
-	 * @return array    an array of ids with the keys preserved
+	 * @return array An array of IDs with the keys preserved.
 	 */
 	public static function insertBatch($table, $values, $getids = true) {
 		if (empty($values)) {
 			return true;
 		}
 
-		$resultIds = [];
-
+		$resultids = [];
 		$table_schema = self::getSchema($table);
-
-		if ($getids) {
-			$id = self::reserveIds($table, count($values));
-		}
-
 		$mandatory_fields = self::getMandatoryFields($table_schema);
 
-		foreach ($values as $key => &$row) {
-			if ($getids) {
-				$resultIds[$key] = $id;
-				$row[$table_schema['key']] = $id;
-				$id = bcadd($id, 1, 0);
-			}
+		if ($getids) {
+			$resultids = self::addIds($table, $values);
+		}
 
+		foreach ($values as &$row) {
 			$row += $mandatory_fields;
 
 			self::checkValueTypes($table_schema, $row);
@@ -596,7 +643,7 @@ class DB {
 			self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s".', $sql));
 		}
 
-		return $resultIds;
+		return $resultids;
 	}
 
 	/**
@@ -943,6 +990,9 @@ class DB {
 			'output' => [],
 			'countOutput' => false,
 			'filter' => [],
+			'search' => [],
+			'startSearch' => false,
+			'searchByAny' => false,
 			'sortfield' => [],
 			'sortorder' => [],
 			'limit' => null,
@@ -976,20 +1026,25 @@ class DB {
 	 * @return array
 	 */
 	public static function select($table_name, array $options, $table_alias = null) {
+		$db_result = DBSelect(self::makeSql($table_name, $options, $table_alias), $options['limit']);
+
+		if ($options['countOutput']) {
+			return DBfetch($db_result)['rowscount'];
+		}
+
 		$result = [];
 		$field_names = array_flip($options['output']);
-		$db_result = DBSelect(self::makeSql($table_name, $options, $table_alias), $options['limit']);
 
 		if ($options['preservekeys']) {
 			$pk = self::getPk($table_name);
 
 			while ($db_row = DBfetch($db_result)) {
-				$result[$db_row[$pk]] = $options['countOutput'] ? $db_row : array_intersect_key($db_row, $field_names);
+				$result[$db_row[$pk]] = array_intersect_key($db_row, $field_names);
 			}
 		}
 		else {
 			while ($db_row = DBfetch($db_result)) {
-				$result[] = $options['countOutput'] ? $db_row : array_intersect_key($db_row, $field_names);
+				$result[] = array_intersect_key($db_row, $field_names);
 			}
 		}
 
@@ -1043,6 +1098,9 @@ class DB {
 		// add filter options
 		$sql_parts = self::applyQueryFilterOptions($table_name, $options, $table_alias, $sql_parts);
 
+		// add search options
+		$sql_parts = self::applyQuerySearchOptions($table_name, $options, $table_alias, $sql_parts);
+
 		// add sort options
 		$sql_parts = self::applyQuerySortOptions($table_name, $options, $table_alias, $sql_parts);
 
@@ -1052,15 +1110,16 @@ class DB {
 	/**
 	 * Modifies the SQL parts to implement all of the output related options.
 	 *
-	 * @param string $table_name
-	 * @param array  $options
-	 * @param string $table_alias
-	 * @param array  $sql_parts
+	 * @param string      $table_name
+	 * @param array       $options
+	 * @param string|null $table_alias
+	 * @param array       $sql_parts
 	 *
+	 * @throws APIException
+	 * @throws DBException
 	 * @return array
 	 */
-	private static function applyQueryOutputOptions($table_name, array $options, $table_alias = null,
-			array $sql_parts) {
+	private static function applyQueryOutputOptions($table_name, array $options, $table_alias, array $sql_parts) {
 		if ($options['countOutput']) {
 			$sql_parts['select'][] = 'COUNT('.self::fieldId('*', $table_alias).') AS rowscount';
 		}
@@ -1086,17 +1145,17 @@ class DB {
 	}
 
 	/**
-	 * Modifies the SQL parts to implement all of the filter related options.
+	 * Modifies the SQL parts to implement all the filter related options.
 	 *
-	 * @param string $table_name
-	 * @param array  $options
-	 * @param string $table_alias
-	 * @param array  $sql_parts
+	 * @param string      $table_name
+	 * @param array       $options
+	 * @param string|null $table_alias
+	 * @param array       $sql_parts
 	 *
+	 * @throws APIException
 	 * @return array
 	 */
-	private static function applyQueryFilterOptions($table_name, array $options, $table_alias = null,
-			array $sql_parts) {
+	private static function applyQueryFilterOptions($table_name, array $options, $table_alias, array $sql_parts) {
 		$table_schema = self::getSchema($table_name);
 		$pk = self::getPk($table_name);
 		$pk_option = $pk.'s';
@@ -1133,17 +1192,93 @@ class DB {
 		return $sql_parts;
 	}
 
+	/**
+	 * Modifies the SQL parts to implement all of the search related options.
+	 *
+	 * @param string      $table_name
+	 * @param array       $options
+	 * @param array       $options['search']
+	 * @param bool        $options['startSearch']
+	 * @param bool        $options['searchByAny']
+	 * @param string|null $table_alias
+	 * @param array       $sql_parts
+	 *
+	 * @throws APIException
+	 * @throws DBException
+	 * @return array
+	 */
+	private static function applyQuerySearchOptions($table_name, array $options, $table_alias, array $sql_parts) {
+		global $DB;
+
+		$table_schema = DB::getSchema($table_name);
+		$unsupported_types = [self::FIELD_TYPE_INT, self::FIELD_TYPE_ID, self::FIELD_TYPE_FLOAT, self::FIELD_TYPE_UINT,
+			self::FIELD_TYPE_BLOB
+		];
+
+		$start = $options['startSearch'] ? '' : '%';
+		$glue = $options['searchByAny'] ? ' OR ' : ' AND ';
+
+		$search = [];
+
+		foreach ($options['search'] as $field_name => $patterns) {
+			if (!array_key_exists($field_name, $table_schema['fields'])) {
+				self::exception(self::SCHEMA_ERROR,
+					vsprintf('%s: field "%s.%s" does not exist.', [__FUNCTION__, $table_name, $field_name])
+				);
+			}
+
+			$field_schema = $table_schema['fields'][$field_name];
+
+			if (in_array($field_schema['type'], $unsupported_types)) {
+				self::exception(self::SCHEMA_ERROR,
+					vsprintf('%s: field "%s.%s" has an unsupported type.', [__FUNCTION__, $table_name, $field_name])
+				);
+			}
+
+			if ($patterns === null) {
+				continue;
+			}
+
+			foreach ((array) $patterns as $pattern) {
+				// escaping parameter that is about to be used in LIKE statement
+				$pattern = mb_strtoupper(strtr($pattern, ['!' => '!!', '%' => '!%', '_' => '!_']));
+				$pattern = $start.$pattern.'%';
+
+				if ($DB['TYPE'] == ZBX_DB_ORACLE && $field_schema['type'] === DB::FIELD_TYPE_NCLOB
+						&& strlen($pattern) > ORACLE_MAX_STRING_SIZE) {
+					$chunks = zbx_dbstr(DB::chunkMultibyteStr($pattern, ORACLE_MAX_STRING_SIZE));
+					$pattern = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
+				}
+				else {
+					$pattern = zbx_dbstr($pattern);
+				}
+
+				$search[] = 'UPPER('.self::fieldId($field_name, $table_alias).') LIKE '.$pattern." ESCAPE '!'";
+			}
+		}
+
+		if ($search) {
+			$sql_parts['where'][] = ($options['searchByAny'] && count($search) > 1)
+				? '('.implode($glue, $search).')'
+				: implode($glue, $search);
+		}
+
+		return $sql_parts;
+	}
+
 	/**
 	 * Apply filter conditions to sql built query.
 	 *
-	 * @param string $table_name
-	 * @param array  $options
-	 * @param string $table_alias
-	 * @param array  $sql_parts
+	 * @param string      $table_name
+	 * @param array       $options
+	 * @param string|null $table_alias
+	 * @param array       $sql_parts
 	 *
-	 * @return bool
+	 * @throws APIException
+	 * @throws DBException
+	 * @return array
 	 */
-	private static function dbFilter($table_name, $options, $table_alias = null, $sql_parts) {
+	private static function dbFilter($table_name, $options, $table_alias, $sql_parts) {
 		$table_schema = self::getSchema($table_name);
 		$filter = [];
 
@@ -1195,14 +1330,16 @@ class DB {
 	/**
 	 * Modifies the SQL parts to implement all of the sorting related options.
 	 *
-	 * @param string $table_name
-	 * @param array  $options
-	 * @param string $table_alias
-	 * @param array  $sql_parts
+	 * @param string      $table_name
+	 * @param array       $options
+	 * @param string|null $table_alias
+	 * @param array       $sql_parts
 	 *
+	 * @throws APIException
+	 * @throws DBException
 	 * @return array
 	 */
-	private static function applyQuerySortOptions($table_name, array $options, $table_alias = null, array $sql_parts) {
+	private static function applyQuerySortOptions($table_name, array $options, $table_alias, array $sql_parts) {
 		$table_schema = self::getSchema($table_name);
 
 		foreach ($options['sortfield'] as $index => $field_name) {
