@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ abstract class DbBackend {
 	protected $tls_cipher_list = '';
 
 	/**
-	 * Set TLS specific options for db conection.
+	 * Set TLS specific options for db connection.
 	 *
 	 * @param string $key_file       Path to TLS key file.
 	 * @param string $cert_file      Path to TLS cert file.
@@ -124,10 +124,13 @@ abstract class DbBackend {
 			return false;
 		}
 
-		$version = DBfetch(DBselect('SELECT dv.mandatory,dv.optional FROM dbversion dv'));
+		$version = DBfetch(DBselect('SELECT dv.mandatory FROM dbversion dv'));
+
 		if ($version['mandatory'] != ZABBIX_DB_VERSION) {
-			$this->setError(_s('The frontend does not match Zabbix database. Current database version (mandatory/optional): %d/%d. Required mandatory version: %d. Contact your system administrator.',
-				$version['mandatory'], $version['optional'], ZABBIX_DB_VERSION));
+			$this->setError(_s('The Zabbix database version does not match current requirements. Your database version: %1$s. Required version: %2$s. Please contact your system administrator.',
+				$version['mandatory'], ZABBIX_DB_VERSION
+			));
+
 			return false;
 		}
 
@@ -151,8 +154,8 @@ abstract class DbBackend {
 	/**
 	 * Create INSERT SQL query for MySQL, PostgreSQL.
 	 * Creation example:
-	 *	INSERT INTO applications (name,hostid,templateid,applicationid)
-	 *	VALUES ('CPU','10113','13','868'),('Filesystems','10113','5','869'),('General','10113','21','870');
+	 *	INSERT INTO items (itemid,name,key_,type)
+	 *	VALUES ('10201','agent.hostname','agent.hostname',1),('10202','agent.ping','agent.ping',1);
 	 *
 	 * @param string $table
 	 * @param array $fields
