@@ -988,7 +988,7 @@ function make_trigger_details($trigger, $eventid) {
 	$table = (new CTableInfo())
 		->addRow([
 			new CCol(_n('Host', 'Hosts', count($hosts))),
-			new CCol($hostNames)
+			(new CCol($hostNames))->addClass(ZBX_STYLE_WORDBREAK)
 		])
 		->addRow([
 			new CCol(_('Trigger')),
@@ -1142,8 +1142,9 @@ function buildExpressionHtmlTree(array $expressionTree, array &$next, &$letterNu
 
 					$url = (new CLinkAction($element['expression']))
 						->setId($expressionId)
-						->onClick('javascript: copy_expression("'.$expressionId.'", '.$type.');');
+						->onClick('copy_expression(this.id, '.$type.');');
 				}
+
 				$expr = expressionLevelDraw($next, $level);
 				$expr[] = SPACE;
 				$expr[] = bold($letter);
@@ -2047,7 +2048,8 @@ function makeTriggersHostsList(array $triggers_hosts) {
 
 		foreach ($hosts as $host) {
 			$host_name = (new CLinkAction($host['name']))
-				->setMenuPopup(CMenuPopupHelper::getHost($host['hostid']));
+				->setMenuPopup(CMenuPopupHelper::getHost($host['hostid']))
+				->addClass(ZBX_STYLE_WORDBREAK);
 
 			if ($host['maintenance_status'] == HOST_MAINTENANCE_STATUS_ON) {
 				if (array_key_exists($host['maintenanceid'], $db_maintenances)) {
