@@ -21,6 +21,7 @@
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
 $widget = (new CWidget())->setTitle($data['title']);
@@ -58,6 +59,7 @@ $form_list = new CFormList('elements_form_list');
 // append copy types to form list
 $form_list->addRow(new CLabel(_('Target type'), 'copy_type'),
 	(new CRadioButtonList('copy_type', (int) $data['copy_type']))
+		->addValue(_('Template groups'), COPY_TYPE_TO_TEMPLATE_GROUP)
 		->addValue(_('Host groups'), COPY_TYPE_TO_HOST_GROUP)
 		->addValue(_('Hosts'), COPY_TYPE_TO_HOST)
 		->addValue(_('Templates'), COPY_TYPE_TO_TEMPLATE)
@@ -84,3 +86,12 @@ $widget->addItem($form);
 require_once dirname(__FILE__).'/js/configuration.copy.elements.js.php';
 
 $widget->show();
+
+(new CScriptTag('
+	view.init('.json_encode([
+		'form_name' => $form->getName(),
+		'copy_targetids' => $data['copy_targetids']
+	]).');
+'))
+	->setOnDocumentReady()
+	->show();
