@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 $page['title'] = _('Configuration of templates');
 $page['file'] = 'templates.php';
-$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'inputsecret.js', 'macrovalue.js',
-	'class.tab-indicators.js', 'class.tagfilteritem.js'
-];
+$page['scripts'] = ['class.tagfilteritem.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -155,7 +153,7 @@ elseif (hasRequest('templateid') && (hasRequest('clone') || hasRequest('full_clo
 	}
 
 	if ($groupids) {
-		$groups_allowed = API::HostGroup()->get([
+		$groups_allowed = API::TemplateGroup()->get([
 			'output' => [],
 			'groupids' => $groupids,
 			'editable' => true,
@@ -179,15 +177,13 @@ elseif (hasRequest('templateid') && (hasRequest('clone') || hasRequest('full_clo
 				: $value;
 		}, $macros);
 
-		$msg = [
-			'type' => 'error',
-			'message' => _('The cloned template contains user defined macros with type "Secret text". The value and type of these macros were reset.'),
-			'source' => ''
-		];
-
-		echo makeMessageBox(false, [$msg], null, true, false)->addClass(ZBX_STYLE_MSG_WARNING);
+		warning(_('The cloned template contains user defined macros with type "Secret text". The value and type of these macros were reset.'));
 	}
 
+	$macros = array_map(function($macro) {
+		return array_diff_key($macro, array_flip(['hostmacroid']));
+	}, $macros);
+
 	if (hasRequest('clone')) {
 		unset($_REQUEST['templateid']);
 	}
@@ -196,15 +192,20 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	try {
 		DBstart();
 
-		$templateId = getRequest('templateid', 0);
+		foreach ($macros as &$macro) {
+			unset($macro['discovery_state']);
+		}
+		unset($macro);
+
+		$input_templateid = getRequest('templateid', 0);
 		$cloneTemplateId = 0;
 
 		if (getRequest('form') === 'full_clone') {
-			$cloneTemplateId = $templateId;
-			$templateId = 0;
+			$cloneTemplateId = $input_templateid;
+			$input_templateid = 0;
 		}
 
-		if ($templateId == 0) {
+		if ($input_templateid == 0) {
 			$messageSuccess = _('Template added');
 			$messageFailed = _('Cannot add template');
 		}
@@ -225,7 +226,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		}
 
 		if ($new_groups) {
-			$new_groupid = API::HostGroup()->create($new_groups);
+			$new_groupid = API::TemplateGroup()->create($new_groups);
 
 			if (!$new_groupid) {
 				throw new Exception();
@@ -241,41 +242,41 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$templates[] = ['templateid' => $templateid];
 		}
 
-		$templatesClear = getRequest('clear_templates', []);
-		$templatesClear = zbx_toObject($templatesClear, 'templateid');
-		$templateName = getRequest('template_name', '');
+		$template_name = getRequest('template_name', '');
 
 		// create / update template
 		$template = [
-			'host' => $templateName,
-			'name' => getRequest('visiblename', ''),
+			'host' => $template_name,
+			'name' => (getRequest('visiblename', '') === '') ? $template_name : getRequest('visiblename'),
+			'description' => getRequest('description', ''),
 			'groups' => zbx_toObject($groups, 'groupid'),
 			'templates' => $templates,
-			'macros' => $macros,
 			'tags' => $tags,
-			'description' => getRequest('description', '')
+			'macros' => $macros
 		];
 
-		if ($templateId == 0) {
+		if ($input_templateid == 0) {
 			$result = API::Template()->create($template);
 
 			if ($result) {
-				$templateId = reset($result['templateids']);
+				$input_templateid = reset($result['templateids']);
 			}
 			else {
 				throw new Exception();
 			}
 		}
 		else {
-			$template['templateid'] = $templateId;
-			$template['templates_clear'] = $templatesClear;
+			$templates_clear = array_diff(
+				getRequest('clear_templates', []),
+				getRequest('add_templates', [])
+			);
+
+			$template['templateid'] = $input_templateid;
+			$template['templates_clear'] = zbx_toObject($templates_clear, 'templateid');
 
 			$result = API::Template()->update($template);
 
-			if ($result) {
-				add_audit_ext(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_TEMPLATE, $templateId, $templateName, 'hosts', null, null);
-			}
-			else {
+			if (!$result) {
 				throw new Exception();
 			}
 		}
@@ -294,7 +295,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		else if (hasRequest('update')) {
 			$del_valuemapids = API::ValueMap()->get([
 				'output' => [],
-				'hostids' => $templateId,
+				'hostids' => $input_templateid,
 				'preservekeys' => true
 			]);
 		}
@@ -305,7 +306,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				unset($del_valuemapids[$valuemap['valuemapid']]);
 			}
 			else {
-				$ins_valuemaps[] = $valuemap + ['hostid' => $templateId];
+				$ins_valuemaps[] = $valuemap + ['hostid' => $input_templateid];
 			}
 		}
 
@@ -328,28 +329,17 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			 * First copy web scenarios with web items, so that later regular items can use web item as their master
 			 * item.
 			 */
-			if (!copyHttpTests($cloneTemplateId, $templateId)) {
+			if (!copyHttpTests($cloneTemplateId, $input_templateid)) {
 				throw new Exception();
 			}
 
-			if (!copyItems($cloneTemplateId, $templateId)) {
+			if (!copyItems($cloneTemplateId, $input_templateid, true)) {
 				throw new Exception();
 			}
 
 			// copy triggers
-			$dbTriggers = API::Trigger()->get([
-				'output' => ['triggerid'],
-				'hostids' => $cloneTemplateId,
-				'inherited' => false
-			]);
-
-			if ($dbTriggers) {
-				$result &= copyTriggersToHosts(zbx_objectValues($dbTriggers, 'triggerid'),
-						$templateId, $cloneTemplateId);
-
-				if (!$result) {
-					throw new Exception();
-				}
+			if (!copyTriggersToHosts([$input_templateid], $cloneTemplateId)) {
+				throw new Exception();
 			}
 
 			// copy graphs
@@ -360,7 +350,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			foreach ($dbGraphs as $dbGraph) {
-				copyGraphToHost($dbGraph['graphid'], $templateId);
+				copyGraphToHost($dbGraph['graphid'], $input_templateid);
 			}
 
 			// copy discovery rules
@@ -371,10 +361,12 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			if ($dbDiscoveryRules) {
-				$result &= API::DiscoveryRule()->copy([
+				if (!API::DiscoveryRule()->copy([
 					'discoveryids' => zbx_objectValues($dbDiscoveryRules, 'itemid'),
-					'hostids' => [$templateId]
-				]);
+					'hostids' => [$input_templateid]
+				])) {
+					$result = false;
+				}
 
 				if (!$result) {
 					throw new Exception();
@@ -390,7 +382,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			if ($db_template_dashboards) {
-				$db_template_dashboards = CDashboardHelper::prepareForClone($db_template_dashboards, $templateId);
+				$db_template_dashboards = CDashboardHelper::prepareForClone($db_template_dashboards, $input_templateid);
 
 				if (!API::TemplateDashboard()->create($db_template_dashboards)) {
 					throw new Exception();
@@ -412,22 +404,56 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 }
 elseif (hasRequest('templateid') && hasRequest('delete')) {
-	DBstart();
+	try {
+		DBstart();
+
+		$hosts = API::Host()->get([
+			'output' => [],
+			'templateids' => getRequest('templateid'),
+			'preservekeys' => true
+		]);
+
+		if ($hosts) {
+			$result = API::Host()->massRemove([
+				'hostids' => array_keys($hosts),
+				'templateids' => getRequest('templateid')
+			]);
+
+			if (!$result) {
+				throw new Exception();
+			}
+		}
+
+		$templates = API::Template()->get([
+			'output' => [],
+			'parentTemplateids' => getRequest('templateid'),
+			'preservekeys' => true
+		]);
+
+		if ($templates) {
+			$result = API::Template()->massRemove([
+				'templateids' => array_keys($templates),
+				'templateids_link' => getRequest('templateid')
+			]);
+
+			if (!$result) {
+				throw new Exception();
+			}
+		}
 
-	$result = API::Template()->massUpdate([
-		'templates' => zbx_toObject($_REQUEST['templateid'], 'templateid'),
-		'hosts' => []
-	]);
-	if ($result) {
 		$result = API::Template()->delete([getRequest('templateid')]);
-	}
 
-	$result = DBend($result);
+		$result = DBend($result);
+	}
+	catch (Exception $e) {
+		DBend(false);
+	}
 
 	if ($result) {
 		unset($_REQUEST['form'], $_REQUEST['templateid']);
 		uncheckTableRows();
 	}
+
 	unset($_REQUEST['delete']);
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
@@ -446,36 +472,69 @@ elseif (hasRequest('templateid') && hasRequest('delete_and_clear')) {
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
 elseif (hasRequest('templates') && hasRequest('action') && str_in_array(getRequest('action'), ['template.massdelete', 'template.massdeleteclear'])) {
-	$templates = getRequest('templates');
+	try {
+		DBstart();
 
-	DBstart();
+		$templateids = getRequest('templates');
 
-	$result = true;
+		if (getRequest('action') === 'template.massdelete') {
+			$hosts = API::Host()->get([
+				'output' => [],
+				'templateids' => $templateids,
+				'preservekeys' => true
+			]);
 
-	if (getRequest('action') === 'template.massdelete') {
-		$result = API::Template()->massUpdate([
-			'templates' => zbx_toObject($templates, 'templateid'),
-			'hosts' => []
-		]);
+			if ($hosts) {
+				$result = API::Host()->massRemove([
+					'hostids' => array_keys($hosts),
+					'templateids' => $templateids
+				]);
+
+				if (!$result) {
+					throw new Exception();
+				}
+			}
+
+			$templates = API::Template()->get([
+				'output' => [],
+				'parentTemplateids' => $templateids,
+				'preservekeys' => true
+			]);
+
+			if ($templates) {
+				$result = API::Template()->massRemove([
+					'templateids' => array_keys($templates),
+					'templateids_link' => $templateids
+				]);
+
+				if (!$result) {
+					throw new Exception();
+				}
+			}
+		}
+
+		$result = API::Template()->delete($templateids);
+
+		$result = DBend($result);
 	}
-
-	if ($result) {
-		$result = API::Template()->delete($templates);
+	catch (Exception $e) {
+		DBend(false);
 	}
 
-	$result = DBend($result);
-
 	if ($result) {
 		uncheckTableRows();
 	}
 	else {
-		$templateids = API::Template()->get([
+		$templates = API::Template()->get([
 			'output' => [],
-			'templateids' => $templates,
-			'editable' => true
+			'templateids' => $templateids,
+			'editable' => true,
+			'preservekeys' => true
 		]);
-		uncheckTableRows(null, zbx_objectValues($templateids, 'templateid'));
+
+		uncheckTableRows(null, array_keys($templates));
 	}
+
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
 
@@ -499,7 +558,7 @@ if (hasRequest('form')) {
 	if ($data['templateid'] != 0) {
 		$dbTemplates = API::Template()->get([
 			'output' => API_OUTPUT_EXTEND,
-			'selectGroups' => API_OUTPUT_EXTEND,
+			'selectTemplateGroups' => ['groupid'],
 			'selectParentTemplates' => ['templateid', 'name'],
 			'selectMacros' => API_OUTPUT_EXTEND,
 			'selectTags' => ['tag', 'value'],
@@ -582,9 +641,14 @@ if (hasRequest('form')) {
 		$data['macros'][] = $macro;
 	}
 
+	foreach ($data['macros'] as &$macro) {
+		$macro['discovery_state'] = CControllerHostMacrosList::DISCOVERY_STATE_MANUAL;
+	}
+	unset($macro);
+
 	if (!hasRequest('form_refresh')) {
 		if ($data['templateid'] != 0) {
-			$groups = zbx_objectValues($data['dbTemplate']['groups'], 'groupid');
+			$groups = array_column($data['dbTemplate']['templategroups'], 'groupid');
 		}
 		else {
 			$groups = getRequest('groupids', []);
@@ -606,7 +670,7 @@ if (hasRequest('form')) {
 
 	// Groups with R and RW permissions.
 	$groups_all = $groupids
-		? API::HostGroup()->get([
+		? API::TemplateGroup()->get([
 			'output' => ['name'],
 			'groupids' => $groupids,
 			'preservekeys' => true
@@ -615,7 +679,7 @@ if (hasRequest('form')) {
 
 	// Groups with RW permissions.
 	$groups_rw = $groupids && (CWebUser::getType() != USER_TYPE_SUPER_ADMIN)
-		? API::HostGroup()->get([
+		? API::TemplateGroup()->get([
 			'output' => [],
 			'groupids' => $groupids,
 			'editable' => true,
@@ -650,6 +714,29 @@ if (hasRequest('form')) {
 		'add_templates' => array_map('strval', array_keys($data['add_templates']))
 	];
 
+	$data['template_name'] = getRequest('template_name', '');
+	$data['visible_name'] = getRequest('visiblename', '');
+
+	$templateids = getRequest('templates', []);
+	$clear_templates = getRequest('clear_templates', []);
+
+	if ($data['templateid'] != 0 && !hasRequest('form_refresh')) {
+		$data['template_name'] = $data['dbTemplate']['host'];
+		$data['visible_name'] = $data['dbTemplate']['name'];
+
+		// Display empty visible name if equal to host name.
+		if ($data['visible_name'] === $data['template_name']) {
+			$data['visible_name'] = '';
+		}
+
+		$templateids = $data['original_templates'];
+	}
+
+	$clear_templates = array_intersect($clear_templates, array_keys($data['original_templates']));
+	$clear_templates = array_diff($clear_templates, array_keys($templateids));
+
+	$data['clear_templates'] = $clear_templates;
+
 	$view = new CView('configuration.template.edit', $data);
 }
 else {
@@ -716,9 +803,9 @@ else {
 		]), ['templateid' => 'id'])
 		: [];
 
-	// Get host groups.
+	// Get template groups.
 	$filter['groups'] = $filter['groups']
-		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+		? CArrayHelper::renameObjectsKeys(API::TemplateGroup()->get([
 			'output' => ['groupid', 'name'],
 			'groupids' => $filter['groups'],
 			'editable' => true,
@@ -728,7 +815,7 @@ else {
 
 	$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
 	if ($filter_groupids) {
-		$filter_groupids = getSubGroups($filter_groupids);
+		$filter_groupids = getTemplateSubGroups($filter_groupids);
 	}
 
 	// Select templates.
